#!/usr/bin/env python
# -*- coding: utf-8 -*-
#     DStat Interface - An interface for the open hardware DStat potentiostat
#     Copyright (C) 2014  Michael D. M. Dryden -
#     Wheeler Microfluidics Laboratory <http://microfluidics.utoronto.ca>
#
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import subprocess
import sys
import os
import time
import logging
from tempfile import mkdtemp
from zipfile import ZipFile

if sys.version_info >= (3,):
    import urllib.request as urllib2
    import urllib.parse as urlparse
else:
    import urllib2
    import urlparse

logger = logging.getLogger(__name__)

try:
    import gi
    gi.require_version('Gtk', '3.0')
    from gi.repository import Gtk, GObject
except ImportError:
    print("ERR: GTK not available")
    sys.exit(1)

import serial

from . import state

url = "http://microfluidics.utoronto.ca/gitlab/api/v4/projects/4/jobs/artifacts/master/download?job=1.2.3&private_token=zkgSx1FaaTP7yLyFKkX6"

def download_fw():  # from https://stackoverflow.com/a/16518224
    temp_dir = mkdtemp()
    logger.info("Temporary directory: {}".format(temp_dir))
    os.chdir(temp_dir)  # Go to temporary directory
    
    u = urllib2.urlopen(url)

    scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
    filename = os.path.basename(path)
    if not filename:
        filename = 'downloaded.file'

    with open(filename, 'wb') as f:
        meta = u.info()
        meta_func = meta.getheaders if hasattr(meta, 'getheaders') else meta.get_all
        meta_length = meta_func("Content-Length")
        file_size = None
        if meta_length:
            file_size = int(meta_length[0])
        logger.info("Downloading: {0} Bytes: {1}".format(url, file_size))

        file_size_dl = 0
        block_sz = 8192
        while True:
            buffer = u.read(block_sz)
            if not buffer:
                break

            file_size_dl += len(buffer)
            f.write(buffer)

            status = "{0:16}".format(file_size_dl)
            if file_size:
                status += "   [{0:6.2f}%]".format(file_size_dl * 100 / file_size)
            status += chr(13)
            logger.info(status)
    
    with ZipFile(filename, mode='r') as z:
        fw_path = z.extract('dstat-firmware.hex')
    
    return fw_path

def test_firmware_version(current=None):
    if current is None:
        current = state.firmware_version
    
    temp_dir = mkdtemp()
    logger.info("Temporary directory: {}".format(temp_dir))
    os.chdir(temp_dir)  # Go to temporary directory
    
    command = "git clone http://microfluidics.utoronto.ca/gitlab/dstat/dstat-firmware.git"
    logger.info('Cloning master.')
    output = subprocess.check_output(command.split(), stderr=subprocess.STDOUT)
    logger.info(output)
    
    os.chdir("./dstat-firmware")
    command = "git merge-base --is-ancestor master {}".format(current)
    test = subprocess.call(command.split())
    
    if test == '0': #already newest
        logger.info('Firmware is latest available.')
        return 'latest'
    if test == '1': #old version
        logger.info('Firmware is out of date.')
        return 'old'
    if test == '128': # newer or different branch
        logger.info('Firmware is not on the master branch.')
        return 'devel'


def dfu_program(path='./dstat-firmware.hex'):
    """Tries to program DStat over USB with DFU with hex file at path."""
    try:
        command = "dfu-programmer atxmega256a3u erase"
        output = subprocess.check_output(command.split(),
                                          stderr=subprocess.STDOUT)
        logger.info(output)
        command = "dfu-programmer atxmega256a3u flash {}".format(path)
        output = subprocess.check_output(command.split(),
                                          stderr=subprocess.STDOUT)
        logger.info(output)
        command = "dfu-programmer atxmega256a3u launch"
        output = subprocess.check_output(command.split(),
                                          stderr=subprocess.STDOUT)
        logger.info(output)
    except subprocess.CalledProcessError as e:
        logger.error("{} failed with output:".format(" ".join(e.cmd)))
        logger.error(e.output)
                                    
def check_dstat(device):
    ser = Serial(device, timeout=1)
    ser.write("ck".encode('ascii'))
    ser.flushInput()
    ser.write('!SF'.encode('ascii'))
    ser.close()
    
    logger.info("DFU command sent to {} at {}".format(port.description, port.device))

# class DFUWindow(object):
#     def __init__(self):
#         def __init__(self):
#                """Adds entry listings to superclass's self.entry dict"""
#                super(ExpInterface, self).__init__() # initialize GObject
#                self.name = "Impedance Spectroscopy"
#
#                self.entry = {} # to be used only for str parameters
#                self._params = None
#
#                self.window = Gtk.ScrolledWindow()
#                self.window.set_vexpand(True)
#
#                grid = Gtk.Grid(orientation=Gtk.Orientation.VERTICAL)
#                grid.set_column_homogeneous(False)
#
#                entries = OrderedDict([
#                           ('start', 'Start (Hz)'),
#                           ('stop', 'Stop (Hz)'),
#                           ('n_increments', 'Steps/Decade'),
#                           ('wait_ms', 'Settling time for first sample (ms)'),
#                           ('cycles', 'Number of settling cycles'),
#                           ('avg_samples', 'Samples to average'),
#                           ('output_mv', "mV p-p"),
#                           ('offset_mv', "Offset (mV)")
#                           ])
#
#                for n, i in enumerate(entries.items()):
#                    key, value = i
#                    grid.attach(Gtk.Label(label=value), 0, n, 1, 1)
#                    self.entry[key] = Gtk.Entry()
#                    grid.attach(self.entry[key], 1, n, 1, 1)
#
#                row = len(entries)
#                grid.attach(Gtk.Label(label="PGA Boost"), 0, row, 1, 1)
#                self.pga_boost = Gtk.CheckButton.new_with_label('5x')
#                grid.attach(self.pga_boost, 1, row, 1, 1)
#                row += 1
#
#                #IV gain
#                self.gain_settings = OrderedDict([
#                                          ('100 V/A', b'1'),
#                                          ('3 kV/A', b'2'),
#                                          ('30 kV/A', b'3'),
#                                          ('300 kV/A', b'4'),
#                                          ('3 MV/A', b'5'),
#                                          ('30 MV/A', b'6'),
#                                          ('100 MV/A', b'7'),
#                                          ('*Bypass', b'0')
#                                     ])
#
#                grid.attach(Gtk.Label(label="IV Gain"), 0, row, 1, 1)
#                self.iv_gain = Gtk.ComboBoxText()
#                for label, cmd in self.gain_settings.items():
#                    self.iv_gain.append(id=cmd, text=label)
#                grid.attach(self.iv_gain, 1, row, 1, 1)
#                row += 1
#
#                #Output Range
#                self.range_settings = OrderedDict([
#                                          ('1x', b'1'),
#                                          ('1/2x', b'2'),
#                                          ('1/5x', b'3'),
#                                          ('1/10x', b'4')
#                                     ])
#
#                grid.attach(Gtk.Label(label="AD5933 Output"), 0, row, 1, 1)
#                self.output_range = Gtk.ComboBoxText()
#                for label, cmd in self.range_settings.items():
#                    self.output_range.append(id=cmd, text=label)
#                grid.attach(self.output_range, 1, row, 1, 1)
#                row += 1
#
#                #Reference
#                self.ref_settings = OrderedDict([
#                                      ('RE Input', b'0'),
#                                      ('Cal 1 (3 kΩ)', b'1'),
#                                      ('Cal 2 (3 MΩ)', b'2'),
#                                      ('Cal 1+2', b'3')
#                                    ])
#
#                grid.attach(Gtk.Label(label="Reference Connection"), 0, row, 1, 1)
#                self.ref_connection = Gtk.ComboBoxText()
#                for label, cmd in self.ref_settings.items():
#                    self.ref_connection.append(id=cmd, text=label)
#                grid.attach(self.ref_connection, 1, row, 1, 1)
#                row += 1
        


if __name__ == "__main__":
    log_handler = logging.StreamHandler()
    log_formatter = logging.Formatter(
                        fmt='%(asctime)s %(levelname)s: [%(name)s] %(message)s',
                        datefmt='%H:%M:%S'
                    )
    log_handler.setFormatter(log_formatter)
    logger.setLevel(level=logging.INFO)
    logger.addHandler(log_handler)
        
    check_dstat()
    time.sleep(2)
    dfu_program(sys.argv[1])